@@ -117,7 +117,6 @@ int yr_atoms_heuristic_quality(
 
   int quality = 0;
   int unique_bytes = 0;
-  int masked_nibbles = 0;
   int i;
 
   assert(atom->length <= YR_MAX_ATOM_LENGTH);
@@ -129,15 +128,13 @@ int yr_atoms_heuristic_quality(
     switch (atom->mask[i])
     {
       case 0x00:
-        masked_nibbles += 2;
+        quality -= 6;
         break;
       case 0x0F:
-        masked_nibbles += 1;
-        quality += 4;
+        quality += 1;
         break;
       case 0xF0:
-        masked_nibbles += 1;
-        quality += 4;
+        quality += 1;
         break;
       case 0xFF:
         switch (atom->bytes[i])
@@ -160,7 +157,7 @@ int yr_atoms_heuristic_quality(
             else
               quality += 20;
         };
-        if (!yr_bitmask_isset(seen_bytes, atom->bytes[i]))
+        if (!yr_bitmask_is_set(seen_bytes, atom->bytes[i]))
         {
           yr_bitmask_set(seen_bytes, atom->bytes[i]);
           unique_bytes++;
@@ -172,16 +169,14 @@ int yr_atoms_heuristic_quality(
   // it heavily.
 
   if (unique_bytes == 1 &&
-      (yr_bitmask_isset(seen_bytes, 0x00) ||
-       yr_bitmask_isset(seen_bytes, 0x20) ||
-       yr_bitmask_isset(seen_bytes, 0xCC) ||
-       yr_bitmask_isset(seen_bytes, 0xFF)))
+      (yr_bitmask_is_set(seen_bytes, 0x00) ||
+       yr_bitmask_is_set(seen_bytes, 0x20) ||
+       yr_bitmask_is_set(seen_bytes, 0xCC) ||
+       yr_bitmask_is_set(seen_bytes, 0xFF)))
   {
     quality -= 10 * atom->length;
   }
 
-  quality -= masked_nibbles * 3;
-
   return YR_MAX_ATOM_QUALITY - 20 * YR_MAX_ATOM_LENGTH + quality;
 }
 
@@ -506,7 +501,7 @@ int _yr_atoms_trim(
   if (atom->length == 0)
     return 0;
 
-  // At this point the actual atom goes from i to i + atom->length and the
+  // The trimmed atom goes from trim_left to trim_left + atom->length and the
   // first and last byte in the atom are known (mask == 0xFF). Now count the
   // number of known and unknown bytes in the atom (mask == 0xFF and
   // mask == 0x00 respectively).
@@ -520,10 +515,10 @@ int _yr_atoms_trim(
   }
 
   // If the number of unknown bytes is >= than the number of known bytes
-  // it doesn't make sense the to use this atom, so we use the a single byte
-  // atom with the first known byte. If YR_MAX_ATOM_LENGTH == 4 this happens
+  // it doesn't make sense the to use this atom, so we use a single byte atpm
+  // containing the first known byte. If YR_MAX_ATOM_LENGTH == 4 this happens
   // only when the atom is like { XX ?? ?? YY }, so using the first known
-  // atom is good enough. For larger values of YR_MAX_ATOM_LENGTH this is not
+  // byte is good enough. For larger values of YR_MAX_ATOM_LENGTH this is not
   // the most efficient solution, as better atoms could be choosen. For
   // example, in { XX ?? ?? ?? YY ZZ } the best atom is { YY ZZ } not { XX }.
   // But let's keep it like this for simplicity.
@@ -586,8 +581,8 @@ static int _yr_atoms_choose(
 
     if (item->atom.length > 0)
     {
-      item->forward_code = node->re_nodes[shift]->forward_code;
-      item->backward_code = node->re_nodes[shift]->backward_code;
+      item->forward_code_ref = node->re_nodes[shift]->forward_code_ref;
+      item->backward_code_ref = node->re_nodes[shift]->backward_code_ref;
       item->backtrack = 0;
       item->next = NULL;
 
@@ -789,8 +784,8 @@ static int _yr_atoms_case_insensitive(
       }
 
       new_atom->atom.length = atom_length;
-      new_atom->forward_code = atom->forward_code;
-      new_atom->backward_code = atom->backward_code;
+      new_atom->forward_code_ref = atom->forward_code_ref;
+      new_atom->backward_code_ref = atom->backward_code_ref;
       new_atom->backtrack = atom->backtrack;
       new_atom->next = *case_insensitive_atoms;
 
@@ -817,6 +812,8 @@ static int _yr_atoms_case_insensitive(
 
 static int _yr_atoms_xor(
     YR_ATOM_LIST_ITEM* atoms,
+    uint8_t min,
+    uint8_t max,
     YR_ATOM_LIST_ITEM** xor_atoms)
 {
   YR_ATOM_LIST_ITEM* atom;
@@ -828,7 +825,7 @@ static int _yr_atoms_xor(
 
   while (atom != NULL)
   {
-    for (j = 1; j <= 255; j++)
+    for (j = min; j <= max; j++)
     {
       new_atom = (YR_ATOM_LIST_ITEM*) yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
 
@@ -842,8 +839,8 @@ static int _yr_atoms_xor(
       }
 
       new_atom->atom.length = yr_min(atom->atom.length, YR_MAX_ATOM_LENGTH);
-      new_atom->forward_code = atom->forward_code;
-      new_atom->backward_code = atom->backward_code;
+      new_atom->forward_code_ref = atom->forward_code_ref;
+      new_atom->backward_code_ref = atom->backward_code_ref;
       new_atom->backtrack = atom->backtrack;
       new_atom->next = *xor_atoms;
 
@@ -898,8 +895,8 @@ static int _yr_atoms_wide(
     }
 
     new_atom->atom.length = yr_min(atom->atom.length * 2, YR_MAX_ATOM_LENGTH);
-    new_atom->forward_code = atom->forward_code;
-    new_atom->backward_code = atom->backward_code;
+    new_atom->forward_code_ref = atom->forward_code_ref;
+    new_atom->backward_code_ref = atom->backward_code_ref;
     new_atom->backtrack = atom->backtrack * 2;
     new_atom->next = *wide_atoms;
 
@@ -980,8 +977,9 @@ static int _yr_atoms_extract_from_re(
 
   FAIL_ON_ERROR(yr_stack_create(1024, sizeof(si), &stack));
 
-  // This first item pushed in the stack is the last one to be poped out, its
-  // sole purpose is forcing that any pending
+  // This first item pushed in the stack is the last one to be poped out, the
+  // sole purpose of this item is forcing that any pending leaf is appended to
+  // current_appending_node during the last iteration of the loop.
   si.re_node = NULL;
   si.new_appending_node = appending_node;
 
@@ -1072,6 +1070,7 @@ static int _yr_atoms_extract_from_re(
                 best_atom_re_nodes[i] = recent_re_nodes[i + shift];
               }
 
+              best_atom.length = atom.length;
               best_quality = quality;
             }
 
@@ -1268,7 +1267,7 @@ static YR_ATOM_LIST_ITEM* _yr_atoms_clone_list_item(
 //
 // Given list of atoms that may contain wildcards, replace those wildcarded
 // atoms with a list of non-wildcarded atoms covering all the combinations
-// allowed by the wilcarded atom. For example, the atom {01 ?2 03} will be
+// allowed by the wildcarded atom. For example, the atom {01 ?2 03} will be
 // replaced by {01 02 03}, {01 12 03}, {01 22 03} .. {01 F2 03}. The list
 // is modified in-place.
 //
@@ -1366,7 +1365,7 @@ static int _yr_atoms_expand_wildcards(
 int yr_atoms_extract_from_re(
     YR_ATOMS_CONFIG* config,
     RE_AST* re_ast,
-    int flags,
+    YR_MODIFIER modifier,
     YR_ATOM_LIST_ITEM** atoms,
     int* min_atom_quality)
 {
@@ -1407,7 +1406,12 @@ int yr_atoms_extract_from_re(
         *atoms = NULL;
       });
 
-  if (flags & STRING_GFLAGS_WIDE)
+  // Don't do convert atoms to wide here if either base64 modifier is used.
+  // This is to avoid the situation where we have "base64 wide" because
+  // the wide has already been applied BEFORE the base64 encoding.
+  if (modifier.flags & STRING_FLAGS_WIDE &&
+      !(modifier.flags & STRING_FLAGS_BASE64 ||
+        modifier.flags & STRING_FLAGS_BASE64_WIDE))
   {
     FAIL_ON_ERROR_WITH_CLEANUP(
         _yr_atoms_wide(*atoms, &wide_atoms),
@@ -1417,7 +1421,7 @@ int yr_atoms_extract_from_re(
           *atoms = NULL;
         });
 
-    if (flags & STRING_GFLAGS_ASCII)
+    if (modifier.flags & STRING_FLAGS_ASCII)
     {
       *atoms = _yr_atoms_list_concat(*atoms, wide_atoms);
     }
@@ -1428,7 +1432,7 @@ int yr_atoms_extract_from_re(
     }
   }
 
-  if (flags & STRING_GFLAGS_NO_CASE)
+  if (modifier.flags & STRING_FLAGS_NO_CASE)
   {
     FAIL_ON_ERROR_WITH_CLEANUP(
         _yr_atoms_case_insensitive(*atoms, &case_insensitive_atoms),
@@ -1452,8 +1456,8 @@ int yr_atoms_extract_from_re(
 
     (*atoms)->atom.length = 0;
     (*atoms)->backtrack = 0;
-    (*atoms)->forward_code = re_ast->root_node->forward_code;
-    (*atoms)->backward_code = NULL;
+    (*atoms)->forward_code_ref = re_ast->root_node->forward_code_ref;
+    (*atoms)->backward_code_ref = YR_ARENA_NULL_REF;
     (*atoms)->next = NULL;
   }
 
@@ -1471,7 +1475,7 @@ int yr_atoms_extract_from_string(
     YR_ATOMS_CONFIG* config,
     uint8_t* string,
     int32_t string_length,
-    int flags,
+    YR_MODIFIER modifier,
     YR_ATOM_LIST_ITEM** atoms,
     int* min_atom_quality)
 {
@@ -1490,8 +1494,8 @@ int yr_atoms_extract_from_string(
   if (item == NULL)
     return ERROR_INSUFFICIENT_MEMORY;
 
-  item->forward_code = NULL;
-  item->backward_code = NULL;
+  item->forward_code_ref = YR_ARENA_NULL_REF;
+  item->backward_code_ref = YR_ARENA_NULL_REF;
   item->next = NULL;
   item->backtrack = 0;
 
@@ -1528,7 +1532,7 @@ int yr_atoms_extract_from_string(
   *atoms = item;
   *min_atom_quality = max_quality;
 
-  if (flags & STRING_GFLAGS_WIDE)
+  if (modifier.flags & STRING_FLAGS_WIDE)
   {
     FAIL_ON_ERROR_WITH_CLEANUP(
         _yr_atoms_wide(*atoms, &wide_atoms),
@@ -1538,7 +1542,7 @@ int yr_atoms_extract_from_string(
           *atoms = NULL;
         });
 
-    if (flags & STRING_GFLAGS_ASCII)
+    if (modifier.flags & STRING_FLAGS_ASCII)
     {
       *atoms = _yr_atoms_list_concat(*atoms, wide_atoms);
     }
@@ -1549,7 +1553,7 @@ int yr_atoms_extract_from_string(
     }
   }
 
-  if (flags & STRING_GFLAGS_NO_CASE)
+  if (modifier.flags & STRING_FLAGS_NO_CASE)
   {
     FAIL_ON_ERROR_WITH_CLEANUP(
         _yr_atoms_case_insensitive(*atoms, &case_insensitive_atoms),
@@ -1562,30 +1566,30 @@ int yr_atoms_extract_from_string(
     *atoms = _yr_atoms_list_concat(*atoms, case_insensitive_atoms);
   }
 
-  if (flags & STRING_GFLAGS_XOR)
+  if (modifier.flags & STRING_FLAGS_XOR)
   {
     FAIL_ON_ERROR_WITH_CLEANUP(
-      _yr_atoms_xor(*atoms, &xor_atoms),
+      _yr_atoms_xor(*atoms, modifier.xor_min, modifier.xor_max, &xor_atoms),
       {
         yr_atoms_list_destroy(*atoms);
         yr_atoms_list_destroy(xor_atoms);
         *atoms = NULL;
       });
 
-    if (flags & STRING_GFLAGS_ASCII ||
-        flags & STRING_GFLAGS_WIDE ||
-        flags & STRING_GFLAGS_NO_CASE)
-    {
-      *atoms = _yr_atoms_list_concat(*atoms, xor_atoms);
-    }
-    else
-    {
-      yr_atoms_list_destroy(*atoms);
-      *atoms = xor_atoms;
-    }
+    yr_atoms_list_destroy(*atoms);
+    *atoms = xor_atoms;
 
   }
 
+  // Recheck the atom quality, in case we have just generated some poor atoms.
+  // https://github.com/VirusTotal/yara/issues/1172
+  for (item = *atoms; item != NULL; item = item->next)
+  {
+    quality = config->get_atom_quality(config, &item->atom);
+    if (quality < *min_atom_quality)
+      *min_atom_quality = quality;
+  }
+
   return ERROR_SUCCESS;
 }
 
