@@ -27,8 +27,6 @@ ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <time.h>
-
 #include <yara/stopwatch.h>
 
 #if defined(_WIN32)
@@ -41,18 +39,18 @@ void yr_stopwatch_start(
 }
 
 
-uint64_t yr_stopwatch_elapsed_us(
+uint64_t yr_stopwatch_elapsed_ns(
     YR_STOPWATCH* sw)
 {
   LARGE_INTEGER li;
 
   QueryPerformanceCounter(&li);
 
-  return (li.QuadPart - sw->start.QuadPart) * 1000000L / sw->frequency.QuadPart;
+  return (li.QuadPart - sw->start.QuadPart) * 1000000000ULL / sw->frequency.QuadPart;
 }
 
 
-#elif defined(__MACH__)
+#elif defined(__APPLE__) && defined(__MACH__)
 
 void yr_stopwatch_start(
     YR_STOPWATCH* sw)
@@ -62,14 +60,11 @@ void yr_stopwatch_start(
 }
 
 
-uint64_t yr_stopwatch_elapsed_us(
+uint64_t yr_stopwatch_elapsed_ns(
     YR_STOPWATCH* sw)
 {
-  uint64_t now;
-
-  now = mach_absolute_time();
-  return (now - sw->start) * sw->timebase.numer /
-         (sw->timebase.denom * 1000ULL);
+  uint64_t now = mach_absolute_time();
+  return ((now - sw->start) * sw->timebase.numer) / sw->timebase.denom;
 }
 
 
@@ -93,7 +88,7 @@ void yr_stopwatch_start(
 }
 
 
-uint64_t yr_stopwatch_elapsed_us(
+uint64_t yr_stopwatch_elapsed_ns(
     YR_STOPWATCH* stopwatch)
 {
   struct timespec ts_stop;
@@ -101,7 +96,7 @@ uint64_t yr_stopwatch_elapsed_us(
 
   clock_gettime(CLOCK_MONOTONIC, &ts_stop);
   timespecsub(&ts_stop, &stopwatch->ts_start, &ts_elapsed);
-  return ts_elapsed.tv_sec * 1000000L + ts_elapsed.tv_nsec / 1000;
+  return ts_elapsed.tv_sec * 1000000000ULL + ts_elapsed.tv_nsec;
 }
 
 
@@ -127,7 +122,7 @@ void yr_stopwatch_start(
 }
 
 
-uint64_t yr_stopwatch_elapsed_us(
+uint64_t yr_stopwatch_elapsed_ns(
     YR_STOPWATCH* stopwatch)
 {
   struct timeval tv_stop;
@@ -135,10 +130,7 @@ uint64_t yr_stopwatch_elapsed_us(
 
   gettimeofday(&tv_stop, NULL);
   timevalsub(&tv_stop, &stopwatch->tv_start, &tv_elapsed);
-  return tv_elapsed.tv_sec * 1000000L + tv_elapsed.tv_usec;
+  return tv_elapsed.tv_sec * 1000000000ULL + tv_elapsed.tv_usec * 1000ULL;
 }
 
-
-
-
 #endif
