@@ -189,6 +189,7 @@ void test_heuristic_quality()
   assert_true_expr(q000102   > q000001);
   assert_true_expr(q00010203 > q010203);
   assert_true_expr(q010203   > q0102);
+  assert_true_expr(q010203   > q00000000);
   assert_true_expr(q0102     > q01);
   assert_true_expr(q01X203   > q0102);
   assert_true_expr(q01X203   > q0001);
@@ -567,6 +568,11 @@ void test_atom_choose()
     assert_hex_atoms("{11 ?? 11 ?? 22 33 44 55 66 }", 1, (struct atom[]) {
       {4, {0x22, 0x33, 0x44, 0x55}},
     });
+
+    // Test case for issue #1025
+    assert_hex_atoms("{?? 11 22 33 ?? 55 66 }", 1, (struct atom[]) {
+      {3, {0x11, 0x22, 0x33}},
+    });
 }
 
 
@@ -577,4 +583,7 @@ int main(int argc, char** argv)
   test_table_quality();
   test_heuristic_quality();
   test_atom_choose();
+
+  yr_finalize();
+  return 0;
 }
